#!/bin/bash
#
# CI QA test for kdump-tools
#
# Copyright (C) 2022-2024 Vitaly Chikunov <vt@altlinux.org>.
# SPDX-License-Identifier: GPL-2.0-only

# shellcheck disable=SC2166
[ -d /.host -a -d /.in -a -d /.out ] || {
	echo >&2 "This script is intended to run inside of hasher."
	exit 1
}

[ "$RPM_INSTALL_NAME" = "kdump-tools-checkinstall" ] || {
	echo >&2 "This script is intended to run inside of checkinstall."
	exit 1
}

[ "$(whoami)" = root ] || {
	echo >&2 "This script should be run under root(er)."
	exit 1
}

cd /tmp
set -euxo pipefail

vm_image_prepare()
{
	# shellcheck disable=SC2026
	sed -i  -e '/#KDUMP_CMDLINE_REMOVE/s/#//' \
		-e '/^KDUMP_CMDLINE_REMOVE/s/"$/ quiet"/'\
		/etc/sysconfig/kdump-tools
	vm-create-image /tmp/i
}

vm_run_cleanup()
{
	rm /tmp/i
	rm /tmp/log
	rm -f /tmp/vm.*
	rm -f /tmp/initramfs-*.img
}

vm_cause_panic()
{
	df -h
	make-initrd
	kdump-config show
	kdump-config load
	kdump-config status
	sync
	kdump-config test
	sleep 1
	echo 1 >/proc/sys/kernel/sysrq
	echo c >/proc/sysrq-trigger
}

# Run inside of another boot of the same VM image to access kdump artifacts.
vm_analyzer()
{
        df -h
        find /var/crash -ls
        kdumpctl list
        file /var/crash/*/kdump
        file /var/crash/*/kdump | grep "Kdump compressed dump" ||
	grep -zq '^KDUMP   ' /var/crash/*/kdump
        kdumpctl dmesg | grep "Kdump: loaded"
        find /var/crash -type f -perm /77 -ls -exec false {} +
}

vm_run_panic()
{
	timeout 100 vm-run --rootfs=/tmp/i --kvm=only --append='crashkernel=256M' "$0" vm_cause_panic |& tee /tmp/log \
	|| echo 'Failure is expected because of crash.'
	grep "sysrq: Trigger a crash" /tmp/log
	grep "Kernel panic - not syncing: sysrq triggered crash" /tmp/log
	grep "Kdump: loaded" /tmp/log
}

vm_run_analyzer()
{
	timeout 100 vm-run --rootfs=/tmp/i --kvm=only "$0" vm_analyzer
}

if [ $# -eq 0 ]; then
	[ -w /dev/kvm ] || {
		echo >&2 "/dev/kvm is needed to run this script."
		exit 1
	}
	vm_image_prepare
	vm_run_panic
	vm_run_analyzer
	vm_run_cleanup
	: Done
else
	"$@"
fi
